// LED Panel
// Daniel Smith
// www.pagemac.com
// Version 1.2 

#include <Adafruit_NeoPixel.h>
#include <stdlib.h>
#define PIN 3
#define LED_COUNT 25

#define BUTTON1  4
#define BUTTON2  5
#define BUTTON3  6
#define BUTTON4  7

#define BUZZER   8
#define GPIO     2
#define LIGHT_SENSOR    A0  //A0

#define SERIAL_BUF_SIZE 64

#define COLOR_RED     0
#define COLOR_GREEN   1
#define COLOR_YELLOW  2
#define COLOR_BLUE    3
#define COLOR_PURPLE  4
#define COLOR_TEAL    5


#define MAX_STRING 20

#define ASCIIdigit(byte1)    (byte1 - '0')
#define ASCIIhex(byte1, byte2) (((byte1 < 'A') ? byte1 - '0' : byte1 - 'A' + 10) << 4) +    \
                                ((byte2 < 'A') ? byte2 - '0' : byte2 - 'A' + 10)
#define ASCIIinthex(byte1, byte2, byte3, byte4)  ((((byte1 < 'A') ? byte1 - '0' : byte1 - 'A' + 10) << 12) +    (((byte2 < 'A') ? byte2 - '0' : byte2 - 'A' + 10) << 8) +   (((byte3 < 'A') ? byte3 - '0' : byte3 - 'A' + 10) << 4) +  ((byte4 < 'A') ? byte4 - '0' : byte4 - 'A' + 10))     
                                
#define MAX_IMG 4
#define MAX_BEEP 2

Adafruit_NeoPixel leds = Adafruit_NeoPixel(LED_COUNT, PIN, NEO_GRB + NEO_KHZ800);

#define NUM_DEMOS 7

#define AMB_MIN_BRIGHTNESS  5
//#define AMB_MAX_BRIGHTNESS  90

// used for ambient light mode
struct LEDdata
{
    unsigned char color;
    unsigned char intensity;
    unsigned char directionUp;
    unsigned char adder;
};

struct LEDdata AmbLEDs[LED_COUNT];

unsigned char ambientModeEnabled=0;
unsigned char ambientModeCount=0;
unsigned char ambientModeTime=20;
unsigned char AMB_MAX_BRIGHTNESS=90;


unsigned char lastButton[5] = {0,0,0,0,0};
unsigned char currentButton[5] = {0,0,0,0,0};

unsigned char codeSequence[5] = {0,0,0,0,0};
unsigned char codeCount=0;

unsigned char myType;  // 0 = none, 1 = right, 2 = left, 3 = top, 4 = bottom, 5 = all
unsigned char myStatus;  // 0 = red, 1 = green, 2 = yellow
unsigned char blinkStatus=0;
unsigned char blinkType=0, blinkStatusColor=0;
volatile unsigned int blinkCount=0;

unsigned long timeoutCounter=0;
unsigned int lightSensorCounter=0;
unsigned int lightSensorAvg, lightSensorAvgCount=0;


unsigned char animationEnabled=0;
unsigned int animateCounter=0;
unsigned int animateFrameDelay = 200;
unsigned char animateColor=0;
unsigned char animateFirstFrame=0, animateCurrentFrame=0, animateLastFrame=0;

unsigned char stringEnabled=0;
unsigned int stringCounter=0;
unsigned int stringFrameDelay = 500;
unsigned char stringColor=0;
unsigned char stringPosition=0;
unsigned char stringData[MAX_STRING] = { 0 };

unsigned char policeBlink=0;
unsigned int policeBlinkCount=0;
unsigned char policeBlinkStage=0;
unsigned char policeBlinkColor1 = COLOR_BLUE;
unsigned char policeBlinkColor2 = COLOR_RED;

unsigned char demoMode=0;
unsigned char demoValue=0;
unsigned char demoColor=0;
unsigned char demoValueMax=0;

volatile unsigned int timer_count=0;

char serial_buf[SERIAL_BUF_SIZE];
int serial_buf_pos=0;

unsigned char image;

unsigned char loopmode=0;
unsigned char loopIntensity=0, loopFlag=0;
unsigned char fixedIntensity=0;

unsigned char beepnum=0;


// printf macro
void p(char *fmt, ... )
{
    char tmp[128]; // resulting string limited to 128 chars
    va_list args;
    va_start (args, fmt );
    vsnprintf(tmp, 128, fmt, args);
    va_end (args);
    Serial.print(tmp);
}

void setPixels (uint32_t color, int num, ...)
{
  // this function simplifies setting multiple LED's to the same color
  // color: the pixel color
  // num: number of pixels to set
  // variable list: pixel number
  va_list args;  
  int i, j;
  va_start(args, num); 
  for (j=0; j < num; j++ )        
  {
      i = va_arg(args, int); 
      leds.setPixelColor(i, color);
      //p("Setting pixel %d to %ul\r\n", i, color);
  }  
}

void setup()
{
  pinMode(BUTTON1, INPUT);   
  pinMode(BUTTON2, INPUT);
  pinMode(BUTTON3, INPUT);
  pinMode(BUTTON4, INPUT);
  
  Serial.begin(9600);
  leds.begin();  
  
  myType = 0;
  myStatus = 0;
  
    // initialize Timer1
  cli();          // disable global interrupts
  TCCR1A = 0;     // set entire TCCR1A register to 0
  TCCR1B = 0;     // same for TCCR1B
   
  OCR1A = 16000;  // set compare match register to desired timer count: (1 ms), timer resolution is 6.25e-8 (16Mhz)
  //TCCR1B |= (1 << WGM12);    // turn on CTC mode:
  TCCR1B |= (1 << CS10);    // no prescaler
  TIMSK1 |= (1 << OCIE1A);  // enable timer compare interrupt:
  sei();                   // enable global interrupts:
  
  loopIntensity = 10;
  
  statusDisplay(myType, myStatus, loopIntensity);
  Serial.print(">?\r");  // what am i???
  if (digitalRead(BUTTON2))
  {
    demoMode = 1;
    demoValue = 8;
    demoColor = 6;
    switchDemoMode();
    tone(BUZZER, 4000, 100);
  }
}

ISR(TIMER1_COMPA_vect)  // this happens every 1.0 ms
{
    timer_count++;
    animateCounter++;
    TCNT1 = 0;
    if (timer_count == 0)  // prevent weird things
    {
      timeoutCounter = 0;
      lightSensorCounter = 0;
      //blinkCounter = 0;
      //animateCounter = 0;
    }
    
    policeBlinkCount++;
    ambientModeCount++;
    stringCounter++;
    blinkCount++;
}


void loop()
{
  unsigned int i;
  unsigned int sensor_value;
  unsigned char a, num, num2, num3;
  unsigned char buttonPressed=0, buttonPressed1=0;
  unsigned long code;
  
  
  uint32_t color1;
  
  buttonPressed = readButtons();     // see if any buttons were pressed
  if (buttonPressed)
  {
    delay(20);    // delay for debouncing
    buttonPressed1 = readButtons();  // read buttons again
    if (buttonPressed1 != buttonPressed) buttonPressed =0;  // same button?  we good!
  }
  
  lastButton[1] = currentButton[1];
  lastButton[2] = currentButton[2];
  lastButton[3] = currentButton[3];
  lastButton[4] = currentButton[4];
  
  if ((buttonPressed > 0) && demoMode)
  {
    
    switch (buttonPressed) {
     case 1:
       demoMode++;
       if (demoMode > NUM_DEMOS) demoMode = 1;
       demoValue = 1;
       break; 
     case 2:
       demoColor++;
       if (demoColor > 6) demoColor = 0;
       break;  
     case 3:
       if (demoValue == 0) 
         demoValue = demoValueMax;
       else 
         demoValue--;
       break;  
     case 4:
       demoValue++;
       if (demoValue >= demoValueMax) demoValue = 0;
       break; 
    }
     switchDemoMode();
     tone(BUZZER, 4000, 100);
  } else if (buttonPressed > 0) // stuff to do when a button is pressed.
  {
    timeoutCounter = timer_count;
    color1 = leds.Color(loopIntensity+10, loopIntensity+10, loopIntensity+10);
    tone(BUZZER, 1200, 100);
    codeSequence[codeCount++] = buttonPressed;
    
    leds.setPixelColor(9+codeCount, color1);
    leds.show();
    delay(50);

    codeDisplay();
    if (codeCount == 5) {
      
        /*
        code = ((unsigned long)codeSequence[0] << 16) + ((unsigned long)codeSequence[1] << 12) +
               ((unsigned long)codeSequence[2] << 8)  + ((unsigned long)codeSequence[3] << 4) + codeSequence[4];
        if (code == 0x11111) {
          blinkStatus=0;  // we can't blink and animate
          animationEnabled = 0;
          statusDisplay(myType, myStatus, loopIntensity);
        } else if (code == 0x22222) {
          animation(1, COLOR_GREEN);
        } else if (code == 0x22221) {
          animation(3, COLOR_RED);
        } else if (code == 0x33333) {
          animation(5, COLOR_BLUE);
        } else if (code == 0x44444) {
          animation(6, COLOR_TEAL);
        }
          */
        p(">C%u%u%u%u%u\r", codeSequence[0], codeSequence[1], codeSequence[2], codeSequence[3], codeSequence[4]);
        codeCount = 0;
        //staticImage(image,loopIntensity);
        clearLEDs();  
        statusDisplay(myType, myStatus, loopIntensity);
    }
    leds.show();
    
  }

  //if ((codeCount > 0) && (timeoutCounter++ == 0x1FFFF))
  if ((codeCount > 0) && (timer_count > (timeoutCounter + 3000)))
  {
    codeCount = 0;
    blockingAnimation(0, myStatus);
    clearLEDs();  
    statusDisplay(myType, myStatus, loopIntensity);
  }
  
  //if (lightSensorCounter++ == 0x8FFF)
  if ((timer_count > (lightSensorCounter + 1000)))
  {
    lightSensorAvg += readLight(); //analogRead(LIGHT_SENSOR);
    if (++lightSensorAvgCount == 4)
    {
        lightSensorAvg = lightSensorAvg >> 2;
        //Serial.println(lightSensorAvg);
        
        if ((!fixedIntensity) && (!codeCount)) setIntensityFromLight(lightSensorAvg);
        lightSensorAvg = lightSensorAvgCount = 0;
    }
    lightSensorCounter = timer_count;
    randomSeed(analogRead(A2));
  }
  
  if (blinkStatus & 1)
  {
    //if (++blinkCounter == 0x2FFF)
    if (blinkCount > 300)
    {
      if (blinkStatus == 1)
      {
        statusDisplay(blinkType, blinkStatusColor, loopIntensity);
      } else {
        statusDisplay(0, 0, loopIntensity);
      }
      codeDisplay();
      blinkStatus = blinkStatus ^ 2;
      blinkCount = 0;
    }
  }
  
  if (animationEnabled) 
  {
    if (animateCounter >  animateFrameDelay)
    {
      clearLEDs(); 
      statusDisplay(animateCurrentFrame, animateColor, loopIntensity);
      codeDisplay();
      
      animateCurrentFrame++;
      if (animateCurrentFrame > animateLastFrame) animateCurrentFrame = animateFirstFrame;
      animateCounter = 0;
    }
  }
  
  if (policeBlink)  // police blink mode
  {
    if (policeBlinkCount > 20)
    {
      if (policeBlinkStage <20) {
        if (policeBlinkStage & 1)  {
            statusDisplay(05, policeBlinkColor1, loopIntensity);  //51
        } else {
          statusDisplay(0,0,0);
        }    
      } else{
        if (policeBlinkStage & 1)  {
            statusDisplay(05, policeBlinkColor2, loopIntensity);  //52
        } else {
          statusDisplay(0,0,0);
        }    
      }
      codeDisplay();
      policeBlinkStage++;
      if (policeBlinkStage > 40)   policeBlinkStage = 0;
      policeBlinkCount = 0;
    }
  }
  
  if (ambientModeEnabled)  // ambient lighting mode
  {
    if (ambientModeCount > ambientModeTime)
    {
      ambientModeStep();
      codeDisplay();
      ambientModeCount = 0;
    }
  }
  
  if (stringEnabled) // string mode
  {

    if (stringCounter >stringFrameDelay)
    {
      clearLEDs();  leds.show();
      delay(20);
      if (stringPosition == 0) { delay(200); }
      statusDisplay(stringData[stringPosition], random(0, 6), loopIntensity);
  
      //p("stringData[%u]: '%c'\r\n", stringPosition, stringData[stringPosition]);
      codeDisplay();
      
      stringPosition++;
      if ((stringData[stringPosition] == 0) || (stringPosition > MAX_STRING)) 
      {
        stringPosition = 0;
      }
      stringCounter = 0;
    }
  }
  
  while (Serial.available() > 0)  // if data in serial RX buffer
  {
    serial_buf[serial_buf_pos++] = Serial.read();    // add byte to our own buffer
    if (serial_buf[serial_buf_pos-1] == '\r' || serial_buf_pos >= SERIAL_BUF_SIZE)  
    {
      switch (serial_buf[0]) 
      {
        case 't':  // set type and status: t1122
          if (serial_buf_pos < 6) goto finish_serial;
          myType = ASCIIhex(serial_buf[1], serial_buf[2]);
          myStatus = ASCIIhex(serial_buf[3], serial_buf[4]);
          clearLEDs();  
          statusDisplay(myType, myStatus, loopIntensity);
          break;
        case 'T':  // set type: T11
          if (serial_buf_pos < 4) goto finish_serial;
          myType = ASCIIhex(serial_buf[1], serial_buf[2]);
          clearLEDs();  
          statusDisplay(myType, myStatus, loopIntensity);
          break;
        case 'S':  // set status color: S11
          if (serial_buf_pos < 4) goto finish_serial;
          myStatus = ASCIIhex(serial_buf[1], serial_buf[2]);
          clearLEDs();  
          statusDisplay(myType, myStatus, loopIntensity);
          break;
        case 'I':  // set a static intensity
          if (serial_buf_pos  < 4) { fixedIntensity = 0; goto finish_serial; }
          num = ASCIIhex(serial_buf[1], serial_buf[2]);
          if (!num) num = 1;
          fadeToIntensity(num);
          fixedIntensity = 1;
          break;
        case 'i':
          fixedIntensity = 0;
          break;
        case 'B':  // start blink mode: B112233\r
                  // 11 is picture num
                  // 22 is color (optional)
                  // 33 is fixed intensity (optional)
          if (serial_buf_pos < 4) goto finish_serial;
          blinkType = ASCIIhex(serial_buf[1], serial_buf[2]);
          if (serial_buf_pos < 6)
            blinkStatusColor = myStatus;      // if we don't have color data, use current status
          else
            blinkStatusColor = ASCIIhex(serial_buf[3], serial_buf[4]);  // color

          stopEverything();
          blinkStatus = 1;
          blinkCount=0;
          statusDisplay(blinkType, blinkStatusColor, loopIntensity);
          if (serial_buf_pos >= 8)  // set fixed intensity if it was provided
          {
            fixedIntensity = ASCIIhex(serial_buf[5], serial_buf[6]);  // set fixed intensity if it was provided
            loopIntensity = fixedIntensity;
            leds.setIntensity(loopIntensity);
            leds.show();
          }
          
         break;
        case 'b': // stop blink
          clearLEDs();
          stopEverything();
          statusDisplay(myType, myStatus, loopIntensity);
          break;
        case 'P':  // display static picture: I1122: 11 is image, 22 is color
          if (serial_buf_pos < 4) goto finish_serial;  // image number
          num = ASCIIhex(serial_buf[1], serial_buf[2]);
          if (serial_buf_pos < 6)
            num2 = myStatus;      // if we don't have color data, use current status
          else
            num2 = ASCIIhex(serial_buf[3], serial_buf[4]);  // color
          stopEverything();
          clearLEDs();
          statusDisplay(num, num2, loopIntensity);
          
          if (serial_buf_pos >= 8)  // set fixed intensity if it was provided
          {
            fixedIntensity = ASCIIhex(serial_buf[5], serial_buf[6]);  // set fixed intensity if it was provided
            loopIntensity = fixedIntensity;
            leds.setIntensity(loopIntensity);
            leds.show();
          }
          break;
        case 'A':  // display short animation
          if (serial_buf_pos < 4) goto finish_serial;
          num = ASCIIhex(serial_buf[1], serial_buf[2]);
          if (serial_buf_pos < 6)
            num2 = myStatus;      // if we don't have color data, use current status
          else
            num2 = ASCIIhex(serial_buf[3], serial_buf[4]);  // color
            
          num3 = fixedIntensity;
          if (serial_buf_pos >= 8)  // set fixed intensity if it was provided
          {
            fixedIntensity = ASCIIhex(serial_buf[5], serial_buf[6]);  // set fixed intensity if it was provided
            loopIntensity = fixedIntensity;
            leds.setIntensity(loopIntensity);
            leds.show();
          }
          
          blockingAnimation(num, num2);
          clearLEDs();  
          fixedIntensity = num3;
          statusDisplay(myType, myStatus, loopIntensity);
          break;
        case 'a':  // display extended animation
          if (serial_buf_pos < 4) { animation(0,0); blinkStatus = 0; goto finish_serial; }
          num = ASCIIhex(serial_buf[1], serial_buf[2]);  // animation number
          if (serial_buf_pos < 6)
            num2 = myStatus;  // if we don't have color data, use current status
          else
            num2 = ASCIIhex(serial_buf[3], serial_buf[4]);  // color
          
          stopEverything();
          animation(num, num2);
          if (serial_buf_pos >= 8)  // set fixed intensity if it was provided
          {
            fixedIntensity = ASCIIhex(serial_buf[5], serial_buf[6]);  // set fixed intensity if it was provided
            loopIntensity = fixedIntensity;
            leds.setIntensity(loopIntensity);
            leds.show();
          }
          break;
        case 'f':  // fix myself
          clearLEDs();
          stopEverything();
          statusDisplay(myType, myStatus, loopIntensity);
          Serial.print(">?\r");  // what am i???
          break;
        case 'G':  // police blink
          stopEverything();
          policeBlink = 1;
          policeBlinkCount = 0;
          policeBlinkStage = 0;
          
          if (serial_buf_pos >= 6) {    // optionally, provide the two colors
            policeBlinkColor1 = ASCIIhex(serial_buf[1], serial_buf[2]);  // animation number
            policeBlinkColor2 = ASCIIhex(serial_buf[3], serial_buf[4]);  // color
          }
          
          if (serial_buf_pos >= 8)  // set fixed intensity if it was provided
          {
            fixedIntensity = ASCIIhex(serial_buf[5], serial_buf[6]);  // set fixed intensity if it was provided
            loopIntensity = fixedIntensity;
            leds.setIntensity(loopIntensity);
            leds.show();
          }
          
          break;
        case 'L':    // ambient lighting mode
          stopEverything();
          fixedIntensity = 1;
          initAmbientMode();
          ambientModeEnabled = 1;
          AMB_MAX_BRIGHTNESS = 90;
          ambientModeTime = 20;
          if (serial_buf_pos >= 4) {    // optionally, provide the max brightness for ambience
            AMB_MAX_BRIGHTNESS = ASCIIhex(serial_buf[1], serial_buf[2]);  
          }
          if (serial_buf_pos >= 6) {    // optionally, provide the ambient mode cycle time (in ms)
            ambientModeTime = ASCIIhex(serial_buf[3], serial_buf[4]);  
          }
          break;
       case 'D':  // start demo mode
          demoMode = 1;
          demoValue = 1;
          demoColor = 2;
          if (serial_buf_pos >= 4) {    // optionally, provide the starting demo type
            demoMode = ASCIIhex(serial_buf[1], serial_buf[2]);  
          }
          if (serial_buf_pos >= 6) {    // optionally, provide the starting demo value
            demoValue = ASCIIhex(serial_buf[3], serial_buf[4]);  
          }
          switchDemoMode();
          break;
        case 'x': // test
          if (serial_buf_pos < 3) goto finish_serial;
          stopEverything();
          serial_buf[serial_buf_pos] = 0;
          serial_buf[serial_buf_pos-1] = 0;
          startCustomString(serial_buf+2, 1);
          break;
        case 'X':
          if (serial_buf_pos < 6) goto finish_serial;
          stringFrameDelay = ASCIIinthex(serial_buf[1], serial_buf[2], serial_buf[3], serial_buf[4]);
          
          break;
        case 'V':  // test
          //setPixels(7, 3, 2, 4, 6);
          //p("animationEnabled = %u\r\n", animationEnabled);
          //p("animateCounter = %u\r\n", animateCounter);
          p("stringCounter = %u\r\n", stringCounter);
          p("stringData[] = %c\r\n", stringData[stringPosition]);
          p("stringPosition = %u\r\n", stringPosition);
          break;
        case 'C':  // command
          if (serial_buf_pos < 4) goto finish_serial;
          num = ASCIIhex(serial_buf[1], serial_buf[2]);
          if (num == 0) {  // rest counters (for sync)
            blinkCount = 0;
            blinkStatus = 1;
          } else if (num == 0x01) {
            num2 = loopIntensity;
            fadeToIntensity(num2+30);
            fadeToIntensity(num2);
          } else if (num == 0x02) {
            fadeToIntensity(loopIntensity+10);
            lightSensorAvg = lightSensorAvgCount = 0; // reset the lighs sensor timer
          } else if (num == 0x10) {   // read light
            num2 = (unsigned char)(((double)readLight() / (double)1024) * (double)100);
            p(">L%02u\r", num2);
          }
        case '-':
          break;
        default:
          p("-!!!\r");
      }
      finish_serial:
      serial_buf_pos = 0;
    }
  }
  
}

void startCustomString(char *str, unsigned char color)
{
  unsigned char i=0;
  i = 0;

  while (i < MAX_STRING)
  {
    stringData[i] = (unsigned char)str[i];
 
    if (str[i] == 0) break;
    i++;
  }

  if (!i) return;
  stringEnabled=1;
  stringCounter=0;
  stringPosition=0;
  stringColor = color;
  
  clearLEDs(); 
  statusDisplay(stringData[stringPosition++], stringColor, loopIntensity);
  

}

void switchDemoMode()
{
  unsigned char mode;
  mode = demoMode;
  stopEverything();
  switch(mode)
  {
    case 0:    // nothing
      break;
    case 1:    // cycle images
      clearLEDs();
      statusDisplay(demoValue, demoColor, loopIntensity);
      demoValueMax = 52;
      break;
    case 2:
      animation(demoValue, demoColor);
      demoValueMax = 6;
      break;
    case 3:
      blinkType = demoValue;
      blinkStatusColor = demoColor;
      blinkStatus = 3;
      demoValueMax = 52;
      animationEnabled = 0;  // animations can't be on at the same time
      break;
    case 4:
      policeBlink = 1;
      policeBlinkCount = 0;
      policeBlinkStage = 0;
      loopIntensity = 70;
      fixedIntensity = 70;
      break;
    case 5:
      fixedIntensity = 1;
      initAmbientMode();
      ambientModeEnabled = 1;
      AMB_MAX_BRIGHTNESS = 90;
      ambientModeTime = 20;
      break;
      
    case 6:
      startCustomString("$%&()*$", 1);
      stringFrameDelay = 1000;
      break;
      
    case 7:
      startCustomString("HELLO WORLD!", 1);
      stringFrameDelay = 250;
      break;
    //default:
      //p("demoMode ? %u\r\n", demoMode);
    
  }
  demoMode = mode;
  /*
  p("demoMode = %u\r\n", demoMode);
  p("demoValue = %u\r\n", demoValue);
  p("demoValueMax = %u\r\n", demoValueMax);
  p("demoColor = %u\r\n", demoColor);
  p("loopIntensity = %u\r\n", loopIntensity);*/
  
}

void initAmbientMode()
{
  unsigned char i;
  for (i=0; i<LED_COUNT; i++)
  {
    AmbLEDs[i].color = random(3, 7);
    AmbLEDs[i].intensity = random(AMB_MIN_BRIGHTNESS, AMB_MAX_BRIGHTNESS);
    AmbLEDs[i].directionUp = random(0, 2);
    AmbLEDs[i].adder = random(1, 3);
    leds.setPixelColor(i, color_lookup(AmbLEDs[i].color, AmbLEDs[i].intensity));
  }
  leds.show();

}

void ambientModeStep()
{
  unsigned char i;
  for (i=0; i<LED_COUNT; i++)
  {
    
    if (AmbLEDs[i].directionUp) 
    {
      AmbLEDs[i].intensity += AmbLEDs[i].adder;
      if (AmbLEDs[i].intensity >= AMB_MAX_BRIGHTNESS) AmbLEDs[i].directionUp = 0;
    } else {
      AmbLEDs[i].intensity -= AmbLEDs[i].adder;
      if (AmbLEDs[i].intensity <= AMB_MIN_BRIGHTNESS) {
        AmbLEDs[i].color = random(3, 7);   
        AmbLEDs[i].adder = random(1, 3); 
        AmbLEDs[i].directionUp = 1;
      }
    }
    
    leds.setPixelColor(i, color_lookup(AmbLEDs[i].color, AmbLEDs[i].intensity));
  }
  leds.show();
}

void stopEverything()
{
  // stop all that shit
    blinkStatus = 0;
    fixedIntensity = 0;
    animationEnabled = 0;
    policeBlink = 0;
    ambientModeEnabled = 0;
    stringEnabled=0;
    demoMode = 0;
}

void setIntensityFromLight(unsigned int lightValue)
{
  // given a current light sensor value (lightValue), calclate a new LED intensity and fade to it
  unsigned char newIntensity;
  if (lightValue > 500) lightValue = 500;
  //newIntensity = (unsigned char) (((float)lightSensorAvg / (float)500) * 80)+1;
  newIntensity = (unsigned char) (((float)lightSensorAvg / (float)500) * 6);
  newIntensity *= 10;
  if (newIntensity == 0) newIntensity = 1;
  
  if (blinkStatus || animationEnabled)  // if blinking or animating, change intensity instantly
  {
      loopIntensity = newIntensity;
      leds.setIntensity(loopIntensity);
      leds.show();
  } else {
    fadeToIntensity(newIntensity);
  }
  
}

void fadeToIntensity(unsigned char newIntensity)
{
  // fade the intensity of LED's from the current intensity (loopIntensity) to newIntensity
  unsigned char i;
  
  char adder=1;  // default to fading up
  if (newIntensity < loopIntensity) adder = -1;  // fade down
  
  while (loopIntensity != newIntensity)
  {
    loopIntensity += adder;
    leds.setIntensity(loopIntensity);
    leds.show();
    delay(7);
  }
          
}

void policeblink()
{
  int i, j;
  for (j=0; j<10; j++)
  {
  for (i=0; i<20; i++)
  {
    statusDisplay(0, 0, 0);
    delay(20);
    statusDisplay(51, COLOR_BLUE, 20);
    delay(20);
  }
  for (i=0; i<20; i++)
  {
    statusDisplay(0, 0, 0);
    delay(20);
    statusDisplay(52, COLOR_RED, 20);
    delay(20);
  }
  }
  
  statusDisplay(0, 0, 0);
}

int readLight()
{
  int value;
  //leds.setIntensity(1);
  //leds.show();
  //delay(1);
  value = analogRead(LIGHT_SENSOR);
  //leds.setIntensity(loopIntensity);
  //leds.show();
  return value;
}

unsigned char readButtons()
{
  unsigned char buttonPressed=0;
  
  currentButton[1] = digitalRead(BUTTON1);
  currentButton[2] = digitalRead(BUTTON2);
  currentButton[3] = digitalRead(BUTTON3);
  currentButton[4] = digitalRead(BUTTON4);

  if (currentButton[1] == 1 && lastButton[1] == 0) {
    buttonPressed = 1;
  } 
  else if (currentButton[2] == 1 && lastButton[2] == 0) {
    buttonPressed = 2;
  }
  else if (currentButton[3] == 1 && lastButton[3] == 0) {
    buttonPressed = 3;
  }
  else if (currentButton[4] == 1 && lastButton[4] == 0) {
    buttonPressed = 4;
  }
  
  return buttonPressed;
}

void serial_flush_buffer()
{
  while (Serial.read() >= 0)
   ; // do nothing
}

// Sets all LEDs to off, bvut DOES NOT update the display;
// call leds.show() to actually turn them off after this.
void clearLEDs()
{
  for (int i=0; i<LED_COUNT; i++)
  {
    leds.setPixelColor(i, 0);
  }
}

uint32_t color_lookup(unsigned char color, unsigned char intensity)
{
  uint32_t color1;

  switch (color) {
    case 0: color1 = leds.Color(intensity, 0, 0); break;  // red
    case 1: color1 = leds.Color(0, intensity, 0); break;  // green
    case 2: color1 = leds.Color(intensity, intensity, 0); break;  // yellow
    case 3: color1 = leds.Color(0, 0, intensity); break;  // blue
    case 4: color1 = leds.Color(intensity, 0, intensity); break;  // purple
    case 5: color1 = leds.Color(0, intensity, intensity); break;  // teal
    case 6: color1 = leds.Color(intensity, intensity, intensity); break;  // white
  }
  
  return color1;
}

void statusDisplay(unsigned char type, unsigned char stat, unsigned char intensity)
{
  uint32_t color1;
  unsigned char i;
  /*
  switch (stat) {
    case 0: color1 = leds.Color(intensity, 0, 0); break;  // red
    case 1: color1 = leds.Color(0, intensity, 0); break;  // green
    case 2: color1 = leds.Color(intensity, intensity, 0); break;  // yellow
    case 3: color1 = leds.Color(0, 0, intensity); break;  // blue
    case 4: color1 = leds.Color(intensity, 0, intensity); break;  // purple
    case 5: color1 = leds.Color(0, intensity, intensity); break;  // teal
    case 6: color1 = leds.Color(intensity, intensity, intensity); break;  // white
  }*/
  color1 = color_lookup(stat, intensity);
  
  switch (type)
  {
    case 0:
      clearLEDs();
      break;
    case 1:  // right bar
      setPixels(color1, 5, 4, 9, 14, 19, 24);
      break;
    case 2:  case '|': // left bar
      setPixels(color1, 5, 0, 5, 10, 15, 20);
      break;
    case 3:  // top bar
      setPixels(color1, 5, 0, 1, 2, 3, 4); 
      break;
    case 4:  case '_': // bottom bar
      setPixels(color1, 5, 20, 21, 22, 23, 24); 
      break;
    case 5:  case '@': // all pixels
      for (i=0; i<25; i++)
       leds.setPixelColor(i, color1);
      break;
    case 6:  // lock symbol
      setPixels(color1, 9, 7, 11, 13, 16, 17, 18, 21, 22, 23);
      break;
    case 7:  // unlock symbol
      setPixels(color1, 10, 2, 6, 8, 13, 16, 17, 18, 21, 22, 23);
      break; 
    case 8:  case ':': // smile :)
      setPixels(color1, 7, 6, 8, 15, 19, 21, 22, 23);
      break;
    case 9: case ';': // frown :(
      setPixels(color1, 7, 6, 8, 16, 17, 18, 20, 24);
      break;
    case 10:  case 'X':// X
      setPixels(color1, 9, 0, 4, 6, 8, 12, 16, 18, 20, 24);
      break;
    case 11:  case 'O': // O
      setPixels(color1, 12, 1, 2, 3, 5, 9, 10, 14, 15, 19, 21, 22, 23);
      break;
    case 12: //case '!':  // !
      setPixels(color1, 4, 2, 7, 12, 22);
      break;
    case 13:  //case '+': // crosshair thing
      setPixels(color1, 12, 0, 1, 3, 4, 5, 9, 15, 19, 20, 21, 23, 24);
      break;
    case 20:  // left arrow animation frame 1
      setPixels(color1, 9, 3, 7, 8, 11, 12, 13, 17, 18, 23);
     break;
    case 21:  // left arrow animation frame 2
      setPixels(color1, 10, 2, 6, 7, 10, 11, 12, 14, 16, 17, 22);
      break;
    case 22:  // left arrow animation frame 3
      setPixels(color1, 12, 1, 5, 6, 9, 10, 11, 13, 14, 15, 16, 19, 21);
      break;
    case 23:  // left arrow animation frame 4
      setPixels(color1, 14, 0, 4, 5, 8, 9, 10, 12, 13, 14, 15, 18, 19, 20, 24);
      break;
    case 24:  // up arrow animation frame 1
      setPixels(color1, 9, 7, 11, 12, 13, 15, 16, 17, 18, 19);
     break;
    case 25:  // up arrow animation frame 2
      setPixels(color1, 10, 2, 6, 7, 8, 10, 11, 12, 13, 14, 22);
      break;
    case 26:  // up arrow animation frame 3
      setPixels(color1, 12, 1, 2, 3, 5, 6, 7, 8, 9, 17, 21, 22, 23);
      break;
    case 27:  // up arrow animation frame 4
      setPixels(color1, 14, 0, 1, 2, 3, 4, 12, 16, 17, 18, 20, 21, 22, 23, 24);
      break;
    case 28:  // right arrow animation frame 1
     setPixels(color1, 9, 1, 6, 7, 11, 12, 13, 16, 17, 21); 
     break;
    case 29:  // right arrow animation frame 2
      setPixels(color1, 10, 2, 7, 8, 10, 12, 13, 14, 17, 18, 22);
      break;
    case 30:  // right arrow animation frame 3
      setPixels(color1, 12, 3, 5, 8, 9, 10, 11, 13, 14, 15, 18, 19, 23);
      break;
    case 31:  // right arrow animation frame 4
      setPixels(color1, 14, 0, 4, 5, 6, 9, 10, 11, 12, 14, 15, 16, 19, 20, 24);
      break;
    case 32:  // down arrow animation frame 1
      setPixels(color1, 9, 5, 6, 7, 8, 9, 11, 12, 13, 17);
     break;
    case 33:  // down arrow animation frame 2
      setPixels(color1, 10, 2, 10, 11, 12, 13, 14, 16, 17, 18, 22);
      break;
    case 34:  // down arrow animation frame 3
      setPixels(color1, 12, 1, 2, 3, 7, 15, 16, 17, 18, 19, 21, 22, 23);
      break;
    case 35:  // down arrow animation frame 4
      setPixels(color1, 14, 0, 1, 2, 3, 4, 6, 7, 8, 12, 20, 21, 22, 23, 24);
      break;
    case 36: // spinner: down
      setPixels(color1, 3, 12, 17, 22);
      break;
    case 37: // spinner: down right
      setPixels(color1, 3, 12, 18, 24);
      break;
    case 38: // spinner: right
      setPixels(color1, 3, 12, 13, 14);
      break;
    case 39: // spinner: right up
      setPixels(color1, 3, 4, 8, 12);
      break;
    case 40: // spinner: up
      setPixels(color1, 3, 2, 7, 12);
      break;
    case 41: // spinner: up left
      setPixels(color1, 3, 0, 6, 12);
      break;
    case 42: // spinner: left
      setPixels(color1, 3, 10, 11, 12);
      break;
    case 43: // spinner: down left
      setPixels(color1, 3, 12, 16, 20);
      break;
    case 44:  // wireless 1
      setPixels(color1, 1, 20);
      break;
    case 45:  // wireless 2
      setPixels(color1, 5, 10, 11, 17, 20, 22);
      break;
    case 46:  // wireless 3
      setPixels(color1, 12, 0, 1, 2, 8, 10, 11, 14, 17, 19, 20, 22, 24);
      break;
    case 47:  // door open 1
      setPixels(color1, 9, 3, 4, 7, 9, 11, 14, 15, 19, 24);
      break;
    case 48:  // door open 2
      setPixels(color1, 10, 3, 4, 7, 9, 12, 14, 16, 19, 21, 24);
      break;
    case 49:  // door open 3
      setPixels(color1, 10, 3, 4, 8, 9, 13, 14, 17, 19, 22, 24);
      break;
    case 50:  // door open 4 (closed)
      setPixels(color1, 10, 3, 4, 8, 9, 13, 14, 18, 19, 23, 24);
      break;
    case 51:  // right 3 rows (for blinking alarm)
      setPixels(color1, 15, 2, 3, 4, 7, 8, 9, 12, 13, 14, 17, 18, 19, 22, 23, 24);
      break;
    case 52:  // left 3 rows (for blinking alarm)
      setPixels(color1, 15, 0, 1, 2, 5, 6, 7, 10, 11, 12, 15, 16, 17, 20, 21, 22);
      break;
      
    case 65:    // A
      setPixels(color1, 14, 1, 2, 3, 5, 9, 10, 14, 15, 16, 17, 18, 19, 20, 24);
      break;
    case 66:    // B
      setPixels(color1, 18, 0, 1, 2, 3, 4, 5, 9, 10, 11, 12, 13, 15, 19, 20, 21, 22, 23, 24);
      break;
    case 67:    // C
      setPixels(color1, 13, 0, 1, 2, 3, 4, 5, 10, 15, 20, 21, 22, 23, 24);
      break;
    case 68:    // D
      setPixels(color1, 14, 0, 1, 2, 3, 5, 9, 10, 14, 15, 19, 20, 21, 22, 23);
      break;
    case 69:    // E
      setPixels(color1, 16, 0, 1, 2, 3, 4, 5, 10, 11, 12, 13, 15, 20, 21, 22, 23, 24);
      break;
    case 70:    // F
      setPixels(color1, 12, 0, 1, 2, 3, 4, 5, 10, 11, 12, 13, 15, 20);
      break;
    case 71:    // G
      setPixels(color1, 16, 0, 1, 2, 3, 4, 5, 10, 13, 14, 15, 19, 20, 21, 22, 23, 24);
      break;
    case 72:    // H
      setPixels(color1, 13, 0, 4, 5, 9, 10, 11, 12, 13, 14, 15, 19, 20, 24);
      break;
    case 73:    // I
      setPixels(color1, 13, 0, 1, 2, 3, 4, 7, 12, 17, 20, 21, 22, 23, 24);
      break;
    case 74:    // J
      setPixels(color1, 10, 4, 9, 14, 15, 19, 20, 21, 22, 23, 24);
      break;
    case 75:    // K
      setPixels(color1, 11, 0, 4, 5, 8, 10, 11, 12, 15, 18, 20, 24);
      break;
    case 76:    // L
      setPixels(color1, 9, 0, 5, 10, 15, 20, 21, 22, 23, 24);
      break;
    case 77:    // M
      setPixels(color1, 13, 0, 4, 5, 6, 8, 9, 10, 12, 14, 15, 19, 20, 24);
      break;
    case 78:    // N
      setPixels(color1, 13, 0, 4, 5, 6, 9, 10, 12, 14, 15, 18, 19, 20, 24);
      break;
    case 80:    // P
      setPixels(color1, 12, 0, 1, 2, 3, 5, 9, 10, 11, 12, 13, 15, 20);
      break;
    case 81:    // Q
      setPixels(color1, 15, 0, 1, 2, 3, 4, 5, 9, 10, 14, 15, 16, 17, 18, 19, 22);
      break;
    case 82:    // R
      setPixels(color1, 14, 0, 1, 2, 3, 5, 9, 10, 11, 12, 13, 15, 18, 20, 24);
      break;
    case 83:    // S
      setPixels(color1, 17, 0, 1, 2, 3, 4, 5, 10, 11, 12, 13, 14, 19, 20, 21, 22, 23, 24);
      break;
    case 84:    // T
      setPixels(color1, 9, 0, 1, 2, 3, 4, 7, 12, 17, 22);
      break;
    case 85:    // U
      setPixels(color1, 13, 0, 4, 5, 9, 10, 14, 15, 19, 20, 21, 22, 23, 24);
      break;
    case 86:    // V
      setPixels(color1, 9, 0, 4, 5, 9, 10, 14, 16, 18, 22);
      break;
    case 87:    // W
      setPixels(color1, 12, 0, 4, 5, 9, 10, 12, 14, 15, 17, 19, 21, 23);
      break;
    case 89:    // Y
      setPixels(color1, 8, 0, 4, 5, 9, 11, 13, 17, 22);
      break;
    case 90:    // Z
      setPixels(color1, 13, 0, 1, 2, 3, 4, 8, 12, 16, 20, 21, 22, 23, 24);
      break;
    //case '#':
      //setPixels(color1, 16, 1, 3, 5, 6, 7, 8, 9, 11, 13, 15, 16, 17, 18, 19, 21, 23);
      break; 
      
    case 128:
      setPixels(color1, 1, 12); 
      break; 
    case 129:
      setPixels(color1, 2, 11, 13);
      break; 
    case 130:
      setPixels(color1, 2, 10, 14);
      break; 
    case 131:
      setPixels(color1, 4, 5, 9, 15, 19);
      break; 
    case 132:
      setPixels(color1, 4, 0, 4, 20, 24);
      break; 
    case 133:
      setPixels(color1, 4, 1, 3, 21, 23);
      break; 
    case 134:
      setPixels(color1, 2, 2, 22);
      break; 
    case 135:
      setPixels(color1, 2, 7, 17);
      break; 
    case 136:
      setPixels(color1, 1, 12);
      break;
    case 137:  case 138: // all pixels
      for (i=0; i<25; i++)
       leds.setPixelColor(i, color1);
      break;
    case 139:
      clearLEDs();
      break;
      
    case 0xFE:  // all pixels
      for (i=0; i<25; i++)
      {
       if (!random(0, 3)) {
       leds.setPixelColor(i, color1);
       }
      }
      break;
    default:  // all pixels
      for (i=0; i<25; i++)
       leds.setPixelColor(i, color_lookup(random(0, 6), loopIntensity));
      break;
  }
  leds.show();
}

void codeDisplay()
{
  uint32_t color;
  int i;
  if (codeCount == 0) return; 
   
  // top and bottom bars
  //color = leds.Color(0, loopIntensity, loopIntensity);  // teal
  //setPixels(color, 10, 5, 6, 7, 8, 9, 15, 16, 17, 18, 19); 
  
  color = leds.Color(2, 1, 2);  // purple
  setPixels(color, 5, 10, 11, 12, 13, 14);  // clear the line
  color = leds.Color(loopIntensity, 0, loopIntensity);  // purple
  
  for (i=0; i < codeCount; i++)
  {
    leds.setPixelColor(i+10, color);
  }
  leds.show();
  //color = leds.Color(loopIntensity, 0, loopIntensity);  // purple
  //leds.setPixelColor(9+codeCount, color);
}

void image_color_freq_time(unsigned char image, unsigned char color, unsigned char intensity, unsigned int freq, unsigned int time)
{
  clearLEDs();
  statusDisplay(image, color, intensity);
  if (freq > 0)
  {
    tone(BUZZER, freq, time);
  }else {
    noTone(BUZZER);
  }
  delay(time);
  
  //clearLEDs();
  //leds.show();
}

void blockingAnimation(unsigned char num, unsigned char color)
{
  unsigned int i, t,f;
  float m;
  
  // color might be ignored! only applies to some animations
  
  switch(num) {
    
  case 0:  // red X denied
    image_color_freq_time(10, COLOR_RED, loopIntensity, 400, 100);
    //delay(100);
    delay(50);
    image_color_freq_time(0, COLOR_RED, loopIntensity, 0, 50);
    image_color_freq_time(10, COLOR_RED, loopIntensity, 400, 300);
    delay(300);
    
    break;
  case 1:  // green 0 approval
    image_color_freq_time(11, COLOR_GREEN, loopIntensity, 400, 100);
    delay(200);
    image_color_freq_time(11, COLOR_GREEN, loopIntensity, 800, 100);
    delay(300);
    
    break;
  case 2:  //  unlock
    image_color_freq_time(6, COLOR_RED, loopIntensity, 400, 100);
    delay(200);
    image_color_freq_time(7, COLOR_GREEN, loopIntensity, 800, 100);
    delay(600);
    
    break;
  case 3:  //  lock
    image_color_freq_time(7, COLOR_GREEN, loopIntensity, 800, 100);
    delay(200);
    image_color_freq_time(6, COLOR_RED, loopIntensity, 400, 100);
    delay(600);
    
    break;
  case 4:  // door opening
     image_color_freq_time(50, color, loopIntensity, 200, 50);
     delay(50);
     image_color_freq_time(49, color, loopIntensity, 300, 50);
     delay(50);
     image_color_freq_time(48, color, loopIntensity, 400, 50);
     delay(50);
     image_color_freq_time(47, color, loopIntensity, 500, 50);
     delay(450);
    break;
  case 5:  // door closing
     image_color_freq_time(47, color, loopIntensity, 500, 50);
     delay(50);
     image_color_freq_time(48, color, loopIntensity, 400, 50);
     delay(50);
     image_color_freq_time(49, color, loopIntensity, 300, 50);
     delay(50);
     image_color_freq_time(50, color, loopIntensity, 200, 50);
     delay(450);
    break;
  case 6:
    clearLEDs();
    statusDisplay(05, color, loopIntensity);
    delay(500);
    clearLEDs();
    leds.show();
    delay(500);
    statusDisplay(05, color, loopIntensity);
    delay(500);
    clearLEDs();
    leds.show();
    delay(500);
    break;
  case 7:
    clearLEDs();
    statusDisplay(05, color, loopIntensity);
    delay(250);
    clearLEDs();
    leds.show();
    delay(250);
    statusDisplay(05, color, loopIntensity);
    delay(250);
    clearLEDs();
    leds.show();
    delay(250);
    break;

  case 0xFF:
    for (i=0; i<10; i++)
    {
      statusDisplay(0xFE, color, loopIntensity);
      delay(10);
      clearLEDs();
    }
    break;
    
  default:
  break;
  }
}

void animation(unsigned char num, unsigned char color)
{
  animationEnabled = 1;
  animateFrameDelay = 200;
  animateColor = color;
  switch (num)
  {
    case 0:
      animationEnabled = 0;
      clearLEDs();
      statusDisplay(myType, myStatus, loopIntensity);
      break;
    case 1:  // arrow left
      animateFirstFrame = animateCurrentFrame = 20;
      animateLastFrame = 23;
      break;
    case 2:  // arrow up
      animateFirstFrame = animateCurrentFrame = 24;
      animateLastFrame = 27;
      break;
    case 3:  // arrow right
      animateFirstFrame = animateCurrentFrame = 28;
      animateLastFrame = 31;
      break;
    case 4:  // arrow down
      animateFirstFrame = animateCurrentFrame = 32;
      animateLastFrame = 35;
      break;
    case 5:  // spinner
      animateFrameDelay = 125;
      animateFirstFrame = animateCurrentFrame = 36;
      animateLastFrame = 43;
      break;
    case 6:  // wireless
      animateFirstFrame = animateCurrentFrame = 44;
      animateLastFrame = 46;
      break;
    case 7:  // thing
      animateFirstFrame = animateCurrentFrame = 128;
      animateLastFrame = 135;
      break;
    case 8:  // red alert
      animateFrameDelay = 500;
      animateFirstFrame = animateCurrentFrame = 137;
      animateLastFrame = 139;
      break;
    case 254:
      animateFrameDelay = 50;
      animateFirstFrame = animateCurrentFrame = 254;
      animateLastFrame = 254;
  }
  animateCounter = timer_count;
}

